@@ -21,8 +21,11 @@ import traceback
 import glob
 import zipfile
 import platform
+import serial
 
+from .util import *
 from .repo import *
+from serial.tools import miniterm
 
 def get_host_os():
     host_os = platform.system()
@@ -41,6 +44,7 @@ def get_host_os():
     return host_os
 
 def cd_aos_root():
+    original_dir = os.getcwd()
     host_os = get_host_os()
     if host_os == 'Win32':
         sys_root = re.compile(r'^[A-Z]{1}:\\$')
@@ -49,9 +53,8 @@ def cd_aos_root():
     while os.path.isdir('./include/aos') == False:
         os.chdir('../')
         if sys_root.match(os.getcwd()):
-            print("error: not in AliOS-Things source code directory")
-            return 'fail'
-    return 'success'
+            return 'fail', original_dir
+    return 'success', original_dir
 
 def which(program):
     if platform.system() == 'Windows' and program.endswith('.exe') == False:
@@ -71,185 +74,142 @@ def which(program):
                 return exe_file
     return None
 
-def download_url(url, file_name=None, retry=1, printlog=True):
+def cmd_version_match(cmd, version):
+    cmd = which(cmd)
+    if cmd == None:
+        return False
+    if version == 'all':
+        return True
+    match = False
+    ret = subprocess.check_output([cmd, '-v'], stderr=subprocess.STDOUT)
+    lines = ret.split('\n')
+    for line in lines:
+        if version in line:
+            match = True
+            break;
+    return match
+
+def download_toolchains(downloads):
+    tmp_dir = 'tmp_{0:02x}'.format(ord(os.urandom(1)))
+    while os.path.isdir(tmp_dir):
+        tmp_dir = 'tmp_{0:02x}'.format(ord(os.urandom(1)))
     try:
-        from urllib.request import urlopen
+        os.mkdir(tmp_dir)
     except:
-        from urllib2 import urlopen
+        print('download toolchains failed: unable to create a temp folder')
+        return
+    try:
+        os.chdir(tmp_dir)
+        for download in downloads:
+            result = 0
+            name, git_url, dst_dir = download
+            print('toolchain {} missing, start download ...'.format(name))
+            print(git_url + ' -> ' + dst_dir)
+            result += subprocess.call(['git', 'clone', '--depth=1', git_url, name])
+            if result > 0:
+                print('git clone toolchain {} failed'.format(name))
+                print('You can mannually try fix this problem by running:')
+                print('    git clone {} {}'.format(git_url, name))
+                print('    mv {0}/main {1} && rm -rf {0}'.format(name, dst_dir))
+            src_dir = name + '/main'
+            if os.path.exists(src_dir) == False:
+                print('toolchain folder {} none exist'.format(src_dir))
+                result += 1
+            if result == 0:
+                dst_dir = '../' + dst_dir
+                if os.path.isfile(dst_dir) == True:
+                    os.remove(dst_dir)
+                if os.path.isdir(dst_dir) == True:
+                    shutil.rmtree(dst_dir)
+                if os.path.isdir(os.path.dirname(dst_dir)) == False:
+                    os.makedirs(os.path.dirname(dst_dir))
+                shutil.move(src_dir, dst_dir)
+                print('download toolchain {} succeed'.format(name))
+            else:
+                print('download toolchain {} failed'.format(name))
+    except:
+        traceback.print_exc()
+    finally:
+        os.chdir('../')
+    try:
+        shutil.rmtree(tmp_dir)
+    except:
+        print("toolchain auto-install error: can not remove temp folder {}, please remove it manually".format(tmp_dir))
+        pass
 
-    if file_name == None:
-        file_name = url.split('/')[-1]
-    succeed = False
-    while retry > 0:
-        u = urlopen(url)
-        f = open(file_name, 'wb')
-        meta = u.info()
-        file_size = int(meta.getheaders("Content-Length")[0])
-
-        file_size_dl = 0
-        block_sz = 8192
-        while True:
-            buffer = u.read(block_sz)
-            if not buffer:
-                break
-
-            file_size_dl += len(buffer)
-            f.write(buffer)
-            if printlog:
-                status = r"Downloading: {0}({1} bytes)  [{2:3.2f}%]".format(file_name, file_size, file_size_dl * 100. / file_size)
-                status = status + chr(8)*(len(status)+1)
-                print(status, end='')
-        f.close()
-        if file_size == file_size_dl:
-            succeed = True
-            break
-        retry -= 1
-    if printlog: print('')
-    return succeed
-
-def __run_setup():
+def _install_toolchains(build_args):
+    board = None
     host_os = get_host_os()
-    sys.stdout.flush()
-    packages = []
-    if host_os.startswith('Linux'):
-        fdevnull = open('/dev/null', 'w')
-        import getpass
-        sudo = 'sudo '
-        if getpass.getuser() == 'root':
-            sudo = ''
-        if which('apt-get') != None:
-            pre_install_cmd = []
-            if host_os.endswith('64') and 'i386' not in subprocess.check_output(['dpkg', '--print-foreign-architectures']):
-               pre_install_cmd += [sudo+'dpkg --add-architecture i386']
-            pre_install_cmd += [sudo+'apt-get -qq update']
-            install_cmd = sudo+'apt-get -yqq install '
-            check_installed_cmd = 'dpkg -s '
-            packages  = ['git', 'wget', 'make', 'flex', 'bison', 'gperf', 'unzip']
-            packages += ['libssl-dev', 'libncurses5-dev', 'libreadline-dev']
-            if host_os.endswith('64'):
-                packages += ['gcc-multilib', 'g++-multilib', 'libssl-dev:i386', 'libncurses5-dev:i386', 'libreadline-dev:i386']
-        elif which('yum') != None:
-            pre_install_cmd = [sudo+'yum -yq update']
-            install_cmd = sudo+'yum -yq install '
-            check_installed_cmd = 'yum list installed '
-            packages  = ['git', 'wget', 'make', 'flex', 'bison', 'gperf', 'unzip']
-            packages += ['gcc', 'gcc-c++', 'glibc-devel', 'libgcc, ''libstdc++-devel']
-            packages += ['openssl-devel', 'ncurses-devel', 'readline-devel']
-            if host_os.endswith('64'):
-                packages += ['glibc-devel.i686', 'libgcc.i686', 'libstdc++-devel.i686']
-                packages += ['openssl-devel.i686', 'ncurses-devel.i686', 'readline-devel.i686']
-        else:
-            print('Warning: not yet supported linux distribution')
-        for package in list(packages):
-            if subprocess.call((check_installed_cmd+package).split(), stdout=fdevnull, stderr=fdevnull) != 0:
+    if host_os == None:
+        error('Unsupported Operating System!')
+
+    #cd to aos root_dir
+    ret, original_dir = cd_aos_root()
+    if ret != 'success':
+        error("not in AliOS-Things source code directory")
+
+    #check config file to be enable this function (for backward compatability)
+    autodownload_enable = False
+    if os.path.exists('build/toolchain_autodownload.config'):
+        try:
+            with open('build/toolchain_autodownload.config') as file:
+                if 'yes' in file.read(): autodownload_enable = True
+        except:
+            pass
+    if autodownload_enable == False:
+        if os.path.isdir(original_dir): os.chdir(original_dir)
+        return
+
+    for arg in build_args:
+        if '@' not in arg:
+            continue
+        args = arg.split('@')
+        board = args[1]
+        break
+
+    if not board:
+        if os.path.isdir(original_dir): os.chdir(original_dir)
+        return
+    if os.path.isdir(os.path.join('board', board)) == False:
+        error('Can not find board {}'.format(board))
+
+    downloads = []
+    from .constant import boards
+    if board in boards:
+        print('Check if required tools for {} exist'.format(board))
+        for toolchain in boards[board]:
+            name = toolchain['name']
+            command = toolchain['command']
+            version = toolchain['version']
+            cmd_path = '{}/{}/bin/{}'.format(toolchain['path'], host_os, command)
+            if cmd_version_match(cmd_path, version) == True:
                 continue
-            packages.remove(package)
+            if toolchain['use_global'] and cmd_version_match(command, version) == True:
+                continue
+            git_url = toolchain['{}_url'.format(host_os)]
+            if git_url == '':
+                continue
+            if is_domestic() is False:
+                git_url = git_url.replace('gitee', 'github')
+                git_url = git_url.replace('alios-things', 'aliosthings')
+            downloads.append([name, git_url, '{}/{}'.format(toolchain['path'], host_os)])
+    if len(downloads): download_toolchains(downloads)
 
-    downloads = {}
-    if os.path.isdir('./build/compiler') == False:
-        toolchains = {
-            'Win32': 'http://alios-things-public.oss-cn-hangzhou.aliyuncs.com/aos-toolchain-windows.zip',
-            'Linux64': 'http://alios-things-public.oss-cn-hangzhou.aliyuncs.com/aos-toolchain-linux64.zip',
-            'Linux32': 'http://alios-things-public.oss-cn-hangzhou.aliyuncs.com/aos-toolchain-linux32.zip',
-            'OSX': 'http://alios-things-public.oss-cn-hangzhou.aliyuncs.com/aos-toolchain-osx.zip'
-            }
-        downloads['toolchain.zip'] = toolchains[host_os]
-    if os.path.isdir('./build/OpenOCD') == False:
-        downloads['openocd.zip'] = 'http://alios-things-public.oss-cn-hangzhou.aliyuncs.com/aos-openocd.zip'
-
-
-    if len(packages) > 0:
-        print("installing dependent packages...")
-        if sudo != '': print("please input your sudo password if asked")
-        print('installing: {0}...'.format(packages), end='')
-        for cmd in pre_install_cmd:
-            popen(cmd, shell=True)
-        sys.stdout.flush()
-        popen(install_cmd + ' '.join(packages), shell=True)
-        print('done')
-
-    remove_tmp_folder = False
-    if os.path.isdir('./tmp') == False:
-        os.mkdir('./tmp')
-        remove_tmp_folder = True
-    os.chdir('./tmp')
-    if host_os == 'Win32':
-        downloads['unzip.zip'] = 'http://alios-things-public.oss-cn-hangzhou.aliyuncs.com/windows-unzip.zip'
-
-    if downloads == {}:
-        if packages == []: print("everything needed is already installed")
-        return 'success'
-
-    for name in downloads:
-        if download_url(downloads[name], name, retry=3) == False:
-            print("error: download {0} failed".format(name))
-            return "fail"
-
-    print('unzip files ...', end='')
-    sys.stdout.flush()
-    if host_os == 'Win32':
-        zfobj = zipfile.ZipFile('unzip.zip')
-        zfobj.extractall('./')
-        zfobj.close()
-        unzip_dir = 'windows-unzip'
-        shutil.move(unzip_dir, 'win-unzip')
-        unzip_program = os.getcwd() + '\\win-unzip\\7za.exe'
-        unzip_pre_opt = ' x '
-        unzip_post_opt = ' -y'
-    else:
-        unzip_program = 'unzip'
-        unzip_pre_opt = ' -qo '
-        unzip_post_opt = ''
-
-    filename = 'toolchain.zip'
-    if filename in downloads:
-        unzip_cmd = unzip_program + unzip_pre_opt + filename + unzip_post_opt
-        ret = os.system(unzip_cmd)
-        if ret != 0:
-            os.chdir('..')
-            print('failed')
-            error('unzip {0} failed'.format(filename))
-            return
-        name = glob.glob('aos-toolchain-*')[0]
-        shutil.move(name + '/compiler', '../build/compiler')
-        shutil.rmtree(name)
-
-    filename = 'openocd.zip'
-    if filename in downloads:
-        unzip_cmd = unzip_program + unzip_pre_opt + filename + unzip_post_opt
-        ret = os.system(unzip_cmd)
-        if ret != 0:
-            os.chdir('..')
-            print('failed')
-            error('unzip {0} failed'.format(filename))
-            return
-        name = 'aos-openocd'
-        shutil.move(name + '/OpenOCD', '../build/OpenOCD')
-        shutil.rmtree(name)
-    print('done')
-
-    print('cleanup...', end='')
-    sys.stdout.flush()
-    if 'unzip.zip' in downloads:
-        shutil.rmtree('win-unzip')
-    for filename in downloads:
-        os.remove(filename)
-    os.chdir('../')
-    if remove_tmp_folder: shutil.rmtree('./tmp', True)
-    print('done')
-    print('succeed')
+    if os.path.isdir(original_dir): os.chdir(original_dir)
 
 def _run_make(arg_list):
+    #install dependent toolchains
+    _install_toolchains(sys.argv[2:])
+
     # check operating system
     host_os = get_host_os()
     if host_os == None:
         error('Unsupported Operating System!')
 
     #cd to aos root_dir
-    original_dir = os.getcwd()
-    ret = cd_aos_root()
+    ret, original_dir = cd_aos_root()
     if ret != 'success':
-        return
+        error("not in AliOS-Things source code directory")
 
     make_cmds = {
         'Win32': 'cmd/win32/make.exe',
@@ -312,33 +272,41 @@ def restore_aos_sdk(aos_sdk_path):
                 pass
 
 
-def generate_aos_program(program_path, aos_sdk_path):
+def generate_aos_program(program_path, aos_sdk_path, template):
     if not(program_path and os.path.isdir(program_path) and aos_sdk_path and os.path.isdir(aos_sdk_path)):
         error("AliOS-Things code isn't exits!!!")
 
+    comp_name = os.path.basename(program_path)
+    template = template if template else 'helloworld'
     pd = Program(program_path)
 
-    program_aos_path = os.path.join(program_path, 'aos')
-    if not os.path.exists(program_aos_path):
-        os.mkdir(program_aos_path)
+    example_dir = os.path.join(aos_sdk_path, 'example')
+    src = os.listdir(os.path.join(example_dir, template))
+    for s in src:
+        if os.path.isdir(s):
+            copy_directory(os.path.join(example_dir, template, s), os.path.join(program_path, s), True)
+        else:
+            shutil.copy(os.path.join(example_dir, template, s), program_path)
 
-    for path in os.listdir(aos_sdk_path):
-        if path != '.git':
-            shutil.move(os.path.join(aos_sdk_path, path), program_aos_path)
+    with open(os.path.join(program_path, comp_name + '.mk'), 'w') as fout:
+        with open(os.path.join(program_path, template + '.mk'), 'r') as fin:
+            for line in fin:
+                if line.startswith('NAME :='):
+                    line = 'NAME := ' + comp_name
+                fout.write(line)
+    os.remove(os.path.join(program_path, template + '.mk'))
 
-    pd.set_cfg(OS_PATH, program_aos_path.replace(os.path.sep, '/'))
-
-    copy_directory(os.path.join(aos_sdk_path, '.git'), os.path.join(program_aos_path, '.git'))
-
-    copy_directory(os.path.join(os.path.dirname(os.path.abspath(__file__)), '.vscode'), os.path.join(program_path, '.vscode'))
-
-    restore_aos_sdk(aos_sdk_path)
-
-    os.mkdir(os.path.join(program_path, 'remote'))
-    pd.set_cfg(REMOTE_PATH, os.path.join(program_path, 'remote').replace(os.path.sep, '/'))
-
-    with open(os.path.join(program_path, 'cube.mk'), 'w') as fout:
-        fout.write('CUBE_AOS_DIR='+program_path.replace(os.path.sep, '/')+'\n')
+    pd.set_cfg(REMOTE_PATH, os.path.join(Global.get_path().replace(os.path.sep, '/'), 'remote'))
+    pd.set_cfg(OS_PATH, aos_sdk_path.replace(os.path.sep, '/'))
+    copy_directory(os.path.join(aos_sdk_path, '.vscode'), os.path.join(program_path, '.vscode'))
+    vscode_cfg_file = os.path.join(program_path, '.vscode', 'settings.json')
+    if os.path.exists(vscode_cfg_file):
+        with open(vscode_cfg_file, 'r+') as f:
+            vscode_cfg = json.load(f)
+            vscode_cfg['aliosStudio.isAosSource'] = False
+            f.seek(0)
+            f.truncate()
+            json.dump(vscode_cfg, f, indent = 4)
 
 
 def get_path_root(path):
@@ -403,6 +371,8 @@ def subcommand(name, *args, **kwargs):
             dict(name='name', help='The program or component name'),
             dict(name=['-c', '--component'], action='store_true',
                  help='Force creation of an aos component. Default: auto.'),
+            dict(name=['-t', '--template'], nargs='?',
+                 help=''),
             dict(name='--scm', nargs='?',
                  help='Source control management. Currently supported: %s. Default: git' % ', '.join(
                      [s.name for s in scms.values()])),
@@ -412,7 +382,7 @@ def subcommand(name, *args, **kwargs):
             description=(
                     "Create a new aos program (aos new helloworld) or component(aos new -c helloworld)\n"
                     "Supported source control management: git"))
-def new(name, scm='git', component=False, protocol=None):
+def new(name, scm='git', component=False, template=None, protocol=None):
     global cwd_root
 
     d_type = 'component' if component else 'program'
@@ -445,13 +415,10 @@ def new(name, scm='git', component=False, protocol=None):
     if d_type == 'program':
         # This helps sub-commands to display relative paths to the created program
         # p.set_root()
-        import requests
-        github_rtt = requests.get('https://raw.githubusercontent.com/alibaba/AliOS-Things/master/README.md').elapsed.total_seconds()
-        gitee_rtt = requests.get('https://gitee.com/alios-things/AliOS-Things/raw/master/README.md').elapsed.total_seconds()
-        if github_rtt < gitee_rtt:
-            aos_os_url = 'https://github.com/alibaba/AliOS-Things.git'
-        else:
+        if is_domestic():
             aos_os_url = 'https://gitee.com/alios-things/AliOS-Things.git'
+        else:
+            aos_os_url = 'https://github.com/alibaba/AliOS-Things.git'
         url = aos_os_url + '#master'
         d = 'aos'
         aos_sdk_path = Global().get_cfg(AOS_SDK_PATH)
@@ -467,7 +434,7 @@ def new(name, scm='git', component=False, protocol=None):
                     rmtree_readonly(os.path.join(d_path, d))
                 raise e
 
-        generate_aos_program(d_path, Global().get_cfg(AOS_SDK_PATH))
+        generate_aos_program(d_path, Global().get_cfg(AOS_SDK_PATH), template)
 
         p.set_cfg(PROGRAM_PATH, d_path.replace(os.path.sep, '/'))
         p.set_cfg(PATH_TYPE, 'program')
@@ -1208,32 +1175,7 @@ def status_(ignore=False):
             with cd(lib.path):
                 status_(ignore)
 
-# Make command
-@subcommand('setup',
-            help='setup and install dependent packages.',
-            description='Setup and install dependent packages.')
-def setup():
-    # check operating system
-    host_os = get_host_os()
-    if host_os == None:
-        error('Unsupported system!')
-
-    #cd to aos root_dir
-    ret = cd_aos_root()
-    if ret != 'success':
-        return
-
-    __run_setup()
-
-
-# Make command
-@subcommand('make',
-            help='Make aos program/component',
-            description="Make aos program/component.")
-def make():
-    # Get the make arguments
-    make_args = ' '.join(sys.argv[2:])
-
+def make_build(make_args):
     # aos new program
     if os.path.isfile(os.path.join(os.getcwd(), Cfg.file)):
         pd = Program(os.getcwd())
@@ -1246,7 +1188,8 @@ def make():
             out_path = os.path.join(os.getcwd(), 'out').replace(os.path.sep, '/')
             build_dir = 'BUILD_DIR=' + out_path
 
-        cube_makefile = 'CUBE_MAKEFILE=' + os.path.join(pd.get_cfg(PROGRAM_PATH), CUBE_MAKEFILE).replace(os.path.sep, '/')
+        # cube_makefile = 'CUBE_MAKEFILE=' + os.path.join(pd.get_cfg(PROGRAM_PATH), CUBE_MAKEFILE).replace(os.path.sep, '/')
+        app_dir = 'APPDIR=' + os.path.dirname(os.getcwd())
 
         with cd(os_path):
             cube_modify = pd.get_cfg(CUBE_MODIFY)
@@ -1256,22 +1199,17 @@ def make():
                     shutil.rmtree(out_path, True)
                 pd.set_cfg(CUBE_MODIFY, '0')
 
-            _run_make(['-e', '-f build/Makefile', make_args, cube_makefile, build_dir])
+            _run_make(['-e', '-f build/Makefile', make_args, app_dir, build_dir])
     else:
         # aos source code
         _run_make(['-e', '-f build/Makefile', make_args])
 
+def scons_build(args):
+    make_args = ['scons -j4 -f ucube.py']
+    target_find = False
 
-# scons make command
-@subcommand('scons',
-            help='Make aos program/component by scons',
-            description="Make aos program/component by scons.")
-def scons():
-    # Get the make arguments
-    args = sys.argv[2:]
-    make_args = ['scons -f ucube.py']
     for arg in args:
-        if '@' in arg:
+        if '@' in arg and not target_find:
             targets = arg.split('@')
             if len(targets) < 2:
                 error('Must special app and board when build aos')
@@ -1282,14 +1220,66 @@ def scons():
             make_args.append(board)
 
             if len(targets) == 3:
-                build_type = 'TYPE=' + targets[3]
+                build_type = 'TYPE=' + targets[2]
                 make_args.append(build_type)
+
+            target_find = True
         else:
             make_args.append(arg)
 
     popen(' '.join(make_args), shell=True, cwd=os.getcwd())
 
 
+def get_scons_enabled_boards():
+    if os.path.exists("build/scons_enabled.py"):
+        module_path = os.path.abspath(os.path.join('build'))
+        sys.path.append(module_path)
+        from scons_enabled import scons_enabled_boards
+        return scons_enabled_boards
+    else:
+        return []
+
+# Make command
+@subcommand('make',
+            help='Make aos program/component',
+            description="Make aos program/component.")
+def make():
+    # Get the make arguments
+    args = sys.argv[2:]
+
+
+    make_args = ' '.join(args)
+    for arg in args:
+        if '@' in arg:
+            targets = arg.split('@')
+            if len(targets) < 2:
+                error('Must special app and board when build aos')
+
+            board = targets[1]
+            if board in get_scons_enabled_boards():
+                scons_build(args)
+            else:
+                make_build(make_args)
+
+            return
+    #aos make clean go here
+    make_build(make_args)
+
+
+# scons make command
+@subcommand('scons',
+            help='Make aos program/component by scons',
+            description="Make aos program/component by scons.")
+def scons():
+    # Get the make arguments
+    args = sys.argv[2:]
+
+    #install dependent toolchains
+    _install_toolchains(args)
+
+    scons_build(args)
+
+
 # Make command
 @subcommand('makelib',
             dict(name='path', nargs='?', help='Library directory path.'),
@@ -1393,58 +1383,219 @@ def config_(var=None, value=None, global_cfg=False, unset=False, list_config=Fal
     else:
         subcommands['config'].error("too few arguments")
 
-# upload command
-@subcommand('upload',
-            help='upload aos image',
-            description="Upload aos image to target platform.")
-def upload():
-    """ FIXME we only support stlink tools for now """
-    target = ' '.join(sys.argv[2:])
-    log("target: %s\n" % target)
-
-    original_dir = os.getcwd()
-    ret = cd_aos_root()
-    if ret != 'success':
-        return
-
-    image_path = os.path.join(os.getcwd(), 'out', target, 'binary', target + '.bin')
-    log("image_path: %s\n" % image_path)
-
+def upload_mk3060(target, aos_path):
     host_os = get_host_os()
     if host_os == None:
         error('Unsupported Operating System!')
+    cmd = os.path.join(aos_path, 'build', 'aos_firmware_update.py').replace('\\', '/')
+    device_port = miniterm.ask_for_port()
+    log('\n[INFO]:cmd: %s\n' % cmd)
+    addr = '0x13200'
+    image_path = os.path.join(os.getcwd(), 'out', target, 'binary', target + '.ota.bin').replace('\\', '/')
+    log("\n[INFO]:image_path: '%s'\n" % image_path)
 
+    exec_cmd = [
+    cmd,
+    device_port, #/dev/ttyUSBx
+    addr,
+    image_path
+    ];
+    log("\n[INFO]:exec_cmd: %s\n" % exec_cmd)
+    if (host_os == 'Win32'):
+        ret = subprocess.call(exec_cmd, shell=True)
+        log("---host_os:%s\n" % host_os)
+    else:
+        ret = subprocess.call(exec_cmd, stdout=sys.stdout, stderr=sys.stderr)
+        log("---host_os:%s\n" % host_os)
+    return ret
+
+def upload_esp32(target, aos_path):
+    host_os = get_host_os()
+    if host_os == None:
+        error('Unsupported Operating System!')
+    cmd = 'esptool.py'
+    port = miniterm.ask_for_port()
+    image_path = os.path.join(os.getcwd(), 'out', target, 'binary', target + '.bin').replace('\\', '/')
+    bootloader_path = os.path.join(aos_path, 'platform/mcu/esp32/bsp/bootloader.bin').replace('\\', '/')
+    partition_path = os.path.join(aos_path, 'platform/mcu/esp32/bsp/custom_partitions.bin').replace('\\', '/')
+    log("\n[INFO]:image_path: %s\n" % image_path)
+    log("[INFO]:bootloader_path: %s\n" % bootloader_path)
+    log("[INFO]:partition_path: %s\n" % partition_path)
+
+    exec_cmd = [
+	cmd,
+	'--chip',
+	'esp32',
+	'--port', port,
+	'--baud', '921600',
+	'--before', 'default_reset',
+	'--after', 'hard_reset',
+	'write_flash',
+	'-z',
+	'--flash_mode',
+	'dio',
+	'--flash_freq',
+	'40m',
+	'--flash_size',
+	'detect',
+	'0x1000', bootloader_path,
+	'0x8000', partition_path,
+	'0x10000', image_path
+    ]
+    log("\n[INFO]:exec_cmd: %s\n" % exec_cmd)
+    if (host_os == 'Win32'):
+        ret = subprocess.call(exec_cmd, shell=True)
+        log("---host_os:%s\n" % host_os)
+    else:
+        ret = subprocess.call(exec_cmd, stdout=sys.stdout, stderr=sys.stderr)
+        log("---host_os:%s\n" % host_os)
+    return ret
+
+def upload_esp8266(target, aos_path):
+    host_os = get_host_os()
+    if host_os == None:
+        error('Unsupported Operating System!')
+    cmd = 'esptool.py'
+    port = miniterm.ask_for_port()
+    image_path = os.path.join(os.getcwd(), 'out', target, 'binary', target + '-0x1000.bin').replace('\\', '/')
+    boot_bin_path = os.path.join(aos_path, 'platform/mcu/esp8266/bsp/boot_v1.7_921600.bin').replace('\\', '/')
+    init_bin_path = os.path.join(aos_path, 'platform/mcu/esp8266/bsp/esp_init_data_default.bin').replace('\\', '/')
+    blank_bin_path = os.path.join(aos_path, 'platform/mcu/esp8266/bsp/blank.bin').replace('\\', '/')
+
+    exec_cmd = [
+	cmd,
+	'--chip', 'esp8266',
+	'--port', port,
+	'--baud', '921600',
+	'write_flash',
+	'--flash_size',
+	'detect',
+	'0x0', boot_bin_path,
+	'0x3fc000', init_bin_path,
+	'0x3fe000', blank_bin_path,
+	'0x1000', image_path
+    ]
+    log("\n[INFO]:exec_cmd: %s\n" % exec_cmd)
+    if (host_os == 'Win32'):
+        ret = subprocess.call(exec_cmd, shell=True)
+        log("---host_os:%s\n" % host_os)
+    else:
+        ret = subprocess.call(exec_cmd, stdout=sys.stdout, stderr=sys.stderr)
+        log("---host_os:%s\n" % host_os)
+    return ret
+
+def upload_stm32(target, aos_path):
     upload_cmds = {
         'Win32': 'win32/st-flash.exe',
         'Linux64': 'linux64/st-flash',
         'Linux32': 'linux32/st-flash',
         'OSX': 'osx/st-flash'
     }
+    board = target.split('@')[1]
+    host_os = get_host_os()
+    if host_os == None:
+        error('Unsupported Operating System!')
 
-    cmds_dir = os.path.join(os.getcwd(), 'build/cmd').replace('\\', '/')
-    upload_cmd = os.path.join(cmds_dir, upload_cmds[host_os])
-    log('upload_cmd: %s\n' % upload_cmd)
+    cmds_dir = os.path.join(aos_path, 'build', 'cmd').replace('\\', '/')
+    upload_cmd = os.path.join(cmds_dir, upload_cmds[host_os]).replace('\\', '/')
+    log("\n[INFO]:upload_cmd: '%s'\n" % upload_cmd)
     addr = '0x08000000'
 
-    erase_cmd = [upload_cmd, 'erase'];
+    if 'starterkit' in board:
+	    # remove unnecessay erase step
+        erase_cmd = [upload_cmd, 'erase']
 
-    ret = subprocess.call(erase_cmd, stdout=sys.stdout, stderr=sys.stderr)
-    if ret == 0:
-        print('erase succeed')
-    else:
-        print('erase failed')
+        ret = subprocess.call(erase_cmd, stdout=sys.stdout, stderr=sys.stderr)
+        if ret == 0:
+            print('erase %s succeed' % board)
+        else:
+            print('erase %s failed' % board)
+
+    image_path = os.path.join(os.getcwd(), 'out', target, 'binary', target + '.bin')
+    log("\n[INFO]:image_path: '%s'\n" % image_path)
 
     exec_cmd = [upload_cmd, '--reset', 'write', image_path, addr];
 
     ret = subprocess.call(exec_cmd, stdout=sys.stdout, stderr=sys.stderr)
+    return ret
+
+def _run_upload(target, aos_path):
+    app = target.split('@')[0]
+    board = target.split('@')[1]
+    ret = None
+
+    if 'esp32' in board:
+        ret = upload_esp32(target, aos_path)
+    elif 'mk3060' in board:
+        ret = upload_mk3060(target, aos_path)
+    elif 'esp8266' in board:
+        ret = upload_esp8266(target, aos_path)
+    else:
+        ret = upload_stm32(target, aos_path)
+    return ret
+
+# upload command
+@subcommand('upload',
+            dict(name='target', help='aos target, e.g. helloworld@starterkit'),
+            help='Upload aos image',
+            description="Upload aos image to target platform.")
+
+def upload(target):
+    """ FIXME we only support stlink tools for now """
+    log("[INFO]:target: %s\n" % target)
+
+    if os.path.isdir('./include/aos') == False:
+        log("[INFO]:Not in aos_sdk_path, curr_path:'%s'\n" % os.getcwd())
+        aos_path = Global().get_cfg(AOS_SDK_PATH)
+        if aos_path == None:
+            error("[ERRO]:Not in aos_sdk_path, aos_sdk unavailable as well. Please run 'aos new $prj_name'!")
+        else:
+            log("[INFO]:Config Loading OK, use '%s' as sdk path\n" % aos_path)
+    else:
+        aos_path = os.getcwd()
+        log("[INFO]:Currently in aos_sdk_path: '%s'\n" % os.getcwd())
+
+    if '@' not in target or len(target.split('@')) != 2:
+        error('target invalid')
+        return;
+
+    ret = _run_upload(target, aos_path)
+
     if ret == 0:
         print('firmware upload succeed')
     else:
         print('firmware upload failed')
 
-    if os.path.isdir(original_dir):
-        os.chdir(original_dir)
 
+@subcommand('monitor',
+            dict(name='port', help='Choose serial port'),
+            dict(name='baud', help='Select baudrate'),
+            help='Serial port monitor\n\n',
+            description="Open a simple serial monitor.")
+def monitor(port, baud):
+    args = ['miniterm', port, baud]
+    host_os = get_host_os()
+    if host_os != 'Win32':
+        args.append('--eol')
+        args.append('LF')
+    sys.argv = args
+    try:
+        miniterm.main()
+    except ProcessException as e:
+        raise e
+
+@subcommand('upgrade',
+            help='Upgrade aos-cube to latest',
+            description="Run pip upgrade process to keep aos-cube up-to-date.")
+def upgrade():
+    cmd = ["pip", "install", "--upgrade", "aos-cube"]
+    try:
+        ret = popen(cmd)
+        if ret != 0:
+            cmd.insert(3, "--no-cache-dir")
+            popen(cmd)
+    except Exception as e:
+        raise e
 
 @subcommand('help',
             help='This help screen')
@@ -1488,6 +1639,7 @@ def main():
             "\"%s\" returned error code %d.\n"
             "Command \"%s\" in \"%s\"" % (e[1], e[0], e[2], e[3]), e[0])
     except OSError as e:
+        traceback.print_exc()
         if e[0] == errno.ENOENT:
             error(
                 "Could not detect one of the command-line tools.\n"
@@ -1498,6 +1650,7 @@ def main():
         info('User aborted!', -1)
         sys.exit(255)
     except Exception as e:
+        traceback.print_exc(file=sys.stdout)
         if very_verbose:
             traceback.print_exc(file=sys.stdout)
         error("Unknown Error: %s" % e, 255)
