@@ -1,8 +1,8 @@
 import ctypes
 
-from config import *
-from util import *
-from repo import *
+from .config import *
+from .util import *
+from .repo import *
 
 
 # Program class, acts code base root
