@@ -2,7 +2,7 @@ import subprocess
 import tempfile
 import errno
 from urlparse import urlparse
-from program import *
+from .program import *
 
 
 def formaturl(url, format="default"):
